@@ -18,14 +18,26 @@ usually represented as the 60th second of the minute during which the
 leap second occurs.
 
 """
+import bisect
 from datetime import datetime
-import warnings
 import argparse
 import subprocess
 
 from dateutil.tz import tzutc, tzlocal
 
-from .__version__ import __version__
+try:
+    from .__version__ import version as __version__
+except ModuleNotFoundError:
+    try:
+        import setuptools_scm
+        __version__ = setuptools_scm.get_version(fallback_version='?.?.?')
+    # FIXME: fallback_version is not available in the buster version
+    # (3.2.0-1)
+    except TypeError:
+        __version__ = setuptools_scm.get_version()
+    except LookupError:
+        __version__ = '?.?.?'
+
 from .leaps import LEAPDATA
 
 
@@ -39,19 +51,17 @@ GPS0 = 315964800
 ##################################################
 
 
+class GPSTimeException(Exception):
+    pass
+
+
 def unix2gps(unix):
     """Convert UNIX timestamp to GPS time.
 
     """
     unix = float(unix)
     gps = unix - GPS0
-    for leap in LEAPDATA:
-        if leap < GPS0:
-            continue
-        if unix < leap:
-            break
-        gps += 1
-    return gps
+    return gps + bisect.bisect(LEAPDATA.as_unix(since_gps_epoch=True), unix)
 
 
 def gps2unix(gps):
@@ -60,48 +70,43 @@ def gps2unix(gps):
     """
     gps = float(gps)
     unix = gps + GPS0
-    for leap in LEAPDATA:
-        if leap < GPS0:
-            continue
-        if unix < leap:
-            break
-        unix -= 1
-    return unix
+    return unix - bisect.bisect(LEAPDATA.as_gps(), gps)
 
 
 ##################################################
 
 
-class GPSTimeException(Exception):
-    pass
-
-
-def cudate(string='now'):
+def _cu_date_parse(string='now'):
     """Parse date/time string to UNIX timestamp with GNU coreutils date
 
     """
     cmd = ['date', '+%s.%N', '-d', string]
     try:
         ts = subprocess.check_output(cmd, stderr=subprocess.PIPE).strip()
-    except subprocess.CalledProcessError:
-        raise GPSTimeException("could not parse string '{}'".format(string))
+    except (FileNotFoundError, subprocess.CalledProcessError) as e:
+        raise GPSTimeException("could not parse string '{}'".format(string)) from e
     return float(ts)
 
 
-def dt2ts(dt):
-    """Return UNIX timestamp for datetime object.
+def _du_date_parse(string='now'):
+    """Parse date/time string to UNIX timestamp with dateutils parse
 
     """
+    from dateutil.parser import parse
     try:
-        dt = dt.astimezone(tzutc())
-        tzero = datetime.fromtimestamp(0, tzutc())
-    except ValueError:
-        warnings.warn("GPS converstion requires timezone info.  Assuming local time...",
-                      RuntimeWarning)
-        dt = dt.replace(tzinfo=tzlocal())
-        tzero = datetime.fromtimestamp(0, tzlocal())
-    delta = dt - tzero
-    return delta.total_seconds()
+        ts = parse(string).timestamp()
+    except Exception as e:
+        raise GPSTimeException("could not parse string '{}'".format(string)) from e
+    return float(ts)
+
+
+# test which date string parser to use.  default to coreutils date if
+# available, since it has a better parser.
+try:
+    _cu_date_parse()
+    _date_parse = _cu_date_parse
+except GPSTimeException:
+    _date_parse = _du_date_parse
 
 
 ##################################################
@@ -131,21 +136,17 @@ class gpstime(datetime):
     1158440653.553765
 
     """
-    def __new__(cls, *args):
-        return datetime.__new__(cls, *args)
-
     @classmethod
     def fromdatetime(cls, datetime):
         """Return gpstime object from datetime object"""
         tzinfo = datetime.tzinfo
         if tzinfo is None:
             tzinfo = tzlocal()
-        cls = gpstime(
+        return cls(
             datetime.year, datetime.month, datetime.day,
             datetime.hour, datetime.minute, datetime.second, datetime.microsecond,
             tzinfo,
         )
-        return cls
 
     @classmethod
     def fromgps(cls, gps):
@@ -181,19 +182,15 @@ class gpstime(datetime):
         if gps is not None:
             return cls.fromgps(gps)
         try:
-            ts = cudate(string)
+            ts = _date_parse(string)
         except GPSTimeException:
             # try again in case this was an ISO string using
             # underscore instead of T as the separator
-            ts = cudate(string.replace('_', 'T'))
+            ts = _date_parse(string.replace('_', 'T'))
         return cls.fromtimestamp(ts).replace(tzinfo=tzlocal())
 
     tconvert = parse
 
-    def timestamp(self):
-        """Return UNIX timestamp (seconds since epoch)."""
-        return dt2ts(self)
-
     def gps(self):
         """Return GPS time as a float."""
         return unix2gps(self.timestamp())
